package com.cz.uhk.bakalarka.app.dao.interfaces;

import com.cz.uhk.bakalarka.app.dao.exception.MinioPayloadException;

import java.io.InputStream;

public interface MessagePayloadDao<T> {

    /**
     * Method saves data to the repository.
     * @param data Data that is saved.
     * @param metadata Metadata from the saved file.
     * @return Metadata from the object in repository that corresponding with saving the file.
     * @throws MinioPayloadException Custom exception that covered all exceptions that repository can throw
     */
    T create(InputStream data,T metadata) throws MinioPayloadException;

    /**
     * Method deletes the file in the repository
     * @param code File is saves with this code. Code identify each file in the repository.  In this case
     *             code identify object, which must be deleted.
     * @throws MinioPayloadException "throw custom exception"
     */
    void delete(String code) throws MinioPayloadException;

    /**
     * Method returns data of the file from repository.
     * @param code  File is saves with this code. Code identify each file in the repository. In this case
     *              code identify object, which´s data must be returned.
     * @return InputStream of the file data from the repository.
     * @throws MinioPayloadException "throw custom exception"
     */
    InputStream getDataByCode(String code) throws MinioPayloadException;

    /**
     * Method returns metadata of the file from repository.
     * @param code File is saves with this code. Code identify each file in the repository. In this case
     *             code identify object, which´s metadata must be returned.
     * @return Metadata object with corresponding metadata from repository.
     * @throws MinioPayloadException "throw custom exception"
     */
    T getByCode(String code) throws MinioPayloadException;


    /**
     * Method that is used for find out if file is saved in repository or not.
     * @param code File is saves with this code. Code identify each file in the repository. In this case
     *             code identify object whose existence need to be decided.
     * @return return decision about exists of file in the repository. It is returned false if file
     *         is not saved in the repository and return true if the file is saved in the repository.
     * @throws MinioPayloadException "throw custom exception"
     */
    boolean exists(String code) throws MinioPayloadException;
}
