package com.cz.uhk.bakalarka.app.service.interfaces;

import com.cz.uhk.bakalarka.app.service.exception.BadCodeFormatException;
import com.cz.uhk.bakalarka.app.service.exception.EmptyFileException;
import com.cz.uhk.bakalarka.app.service.exception.LargeFileException;
import com.cz.uhk.bakalarka.app.service.exception.UnsupportedFileFormatException;
import org.springframework.web.multipart.MultipartFile;

/**
 * Interface that contains method responsible to validate.
 */
public interface InputMessageValidationServiceInf {

    /**
     * Method validate file. Validation aspects are size and file format.
     * @param multipartFile file selected to the validation
     * @return Array with two indexes, first index contain information about file format
     *         and second contain generated code, which is assigned to this file.
     * @throws LargeFileException Exception is thrown if file is larger than is allowed.
     * @throws UnsupportedFileFormatException Exception is thrown if file have unsupported file format.
     * @throws EmptyFileException Exception is thrown if file is empty.
     */
    String[] validateCreate(MultipartFile multipartFile) throws LargeFileException, UnsupportedFileFormatException, EmptyFileException;

    /**
     * Method that validate code correctness with regex pattern.
     * @param code input code to validate.
     * @return true if validation is successful, false if there is error.
     * @throws BadCodeFormatException exception that is thrown if code is in bad pattern.
     */
    boolean validateCodeCorrectness(String code) throws BadCodeFormatException;

}
