package com.cz.uhk.bakalarka.app.service.interfaces;

import com.cz.uhk.bakalarka.app.model.MetaData;
import com.cz.uhk.bakalarka.app.service.exception.PayloadDaoException;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;
import java.io.InputStream;

/**
 * Service layer interface that transfer payload and Metadata from controller to DAO.
 */
public interface MessageServiceInf {

    /**
     * @param payload payload from file.
     * @param metaData metadata of the file.
     * @param sseEmitter sseEmitter that is used to send information about processing.
     * @throws Exception All exception that can be thrown.
     */
    void create(InputStream payload, MetaData metaData, SseEmitter sseEmitter) throws Exception;

    /**
     * @param code code of the file, that is selected to delete.
     * @throws PayloadDaoException Exception that is thrown if is thrown MinioPayloadException.
     */
    void delete(String code) throws PayloadDaoException;


    /**
     * @param code code of the file, which payload must be return.
     * @return InputStream of the payload.
     * @throws PayloadDaoException Exception that is thrown if is thrown MinioPayloadException.
     */
    InputStream getDataByCode(String code) throws PayloadDaoException;

    /**
     * @param code code of the file, which Metadata must be return.
     * @return Metadata of the requested file.
     * @throws PayloadDaoException
     */
    MetaData getByCode(String code) throws PayloadDaoException;
}
