package com.cz.uhk.bakalarka.app.service;

import com.cz.uhk.bakalarka.app.dao.MessagePayloadMinIoDao;
import com.cz.uhk.bakalarka.app.dao.exception.MinioPayloadErrorResponseException;
import com.cz.uhk.bakalarka.app.dao.exception.MinioPayloadException;
import com.cz.uhk.bakalarka.app.dao.interfaces.MessagePayloadDao;
import com.cz.uhk.bakalarka.app.model.MetaData;
import com.cz.uhk.bakalarka.app.service.exception.PayloadDaoErrorResponseException;
import com.cz.uhk.bakalarka.app.service.exception.PayloadDaoException;
import com.cz.uhk.bakalarka.app.support.MetaDataSupport;
import lombok.extern.log4j.Log4j2;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Mockito;
import org.mockito.junit.jupiter.MockitoExtension;
import java.io.ByteArrayInputStream;
import java.io.InputStream;

import static org.junit.jupiter.api.Assertions.*;


@Log4j2
@ExtendWith(MockitoExtension.class)
class MessageServiceTest {


    private MessagePayloadDao<MetaData> messagePayloadDao;
    private MessageService messageService;


    @BeforeEach
    void setUp() {
        messagePayloadDao = Mockito.mock(MessagePayloadMinIoDao.class);
        messageService = new MessageService((MessagePayloadMinIoDao) messagePayloadDao);
    }


    @Test
    void delete_throwingPayloadDaoErrorResponseException() throws Exception{
        log.info("Test to success handle MinioPayloadDaoErrorResponseException in delete method");
        Mockito.doThrow(MinioPayloadErrorResponseException.class).when(messagePayloadDao).delete(Mockito.any());

        log.info("Assert thrown exception and expected exception.");
        assertThrows(PayloadDaoErrorResponseException.class, () -> messageService.delete(Mockito.any()));

    }

    @Test
    void delete_throwingPayloadDaoException() throws Exception{
        log.info("Test to success handle MinioPayloadException in delete method");
        Mockito.doThrow(MinioPayloadException.class).when(messagePayloadDao).delete(Mockito.any());

        log.info("Assert thrown exception and expected exception.");
        assertThrows(PayloadDaoException.class, () -> messageService.delete(Mockito.any()));

    }

    @Test
    void getDataByCode() throws Exception{
        log.info("Test to success return payload from dao");
        InputStream payload = new ByteArrayInputStream(new byte[] {
                118, 122, 111, 114, 111, 118, 101, 74, 115, 111, 110, 80, 114, 111, 71, 97, 116, 101, 119, 97, 121, 46, 106, 115, 111, 110
        });


        Mockito.doReturn(payload).when(messagePayloadDao).getDataByCode("code");

        log.info("Assert expected and returned payload.");
        assertEquals(payload, messageService.getDataByCode("code"));
    }

    @Test
    void getDataByCode_throwingPayloadDaoErrorException() throws Exception {
        log.info("Test to success handle MinioPayloadErrorResponseException in getDataByCode method");
        Mockito.doThrow(MinioPayloadErrorResponseException.class).when(messagePayloadDao).getDataByCode("code");

        log.info("Assert thrown exception and expected exception.");
        assertThrows(PayloadDaoErrorResponseException.class, () -> messageService.getDataByCode("code"));
    }

    @Test
    void getDataByCode_throwingPayloadDaoException() throws Exception {
        log.info("Test to success handle MinioPayloadException in getDataByCode method");
        Mockito.doThrow(MinioPayloadException.class).when(messagePayloadDao).getDataByCode("code");

        log.info("Assert thrown exception and expected exception.");
        assertThrows(PayloadDaoException.class, () -> messageService.getDataByCode("code"));
    }


    @Test
    void getByCode() throws Exception{
        log.info("Test to success return metadata from dao");
        MetaData metaData = MetaDataSupport.getMetaDataWithCode();

        Mockito.doReturn(metaData).when(messagePayloadDao).getByCode(metaData.getCode());

        log.info("Assert expected and returned metadata.");
        assertEquals(metaData, messageService.getByCode(metaData.getCode()));
    }

    @Test
    void getByCode_throwingPayloadDaoErrorResponseException() throws Exception{
        log.info("Test to success handle MinioPayloadErrorResponseException in getByCode method");
        Mockito.doThrow(MinioPayloadErrorResponseException.class).when(messagePayloadDao).getByCode("code");

        assertThrows(PayloadDaoErrorResponseException.class, () -> messageService.getByCode("code"));
    }

    @Test
    void getByCode_throwingPayloadDaoException() throws Exception{
        log.info("Test to success handle MinioPayloadException in getByCode method");
        Mockito.doThrow(MinioPayloadException.class).when(messagePayloadDao).getByCode("code");

        log.info("Assert thrown exception and expected exception.");
        assertThrows(PayloadDaoException.class, () -> messageService.getByCode("code"));
    }
}